import tempfile
from contextlib import ExitStack
from queue import PriorityQueue
from typing import Iterator, TypeVar, Iterable

from tqdm import tqdm

from ciff_toolkit.ciff_pb2 import DocRecord, Header, Posting, PostingsList
from ciff_toolkit.read import CiffReader, DEFAULT_CHUNK_SIZE
from ciff_toolkit.write import CiffWriter

TA = TypeVar('TA')


class CiffMerge:
    def __init__(self, writer: CiffWriter, *readers: CiffReader, show_progress: bool = True):
        self.writer = writer
        self.readers = readers
        self.show_progress = show_progress

        # Saves the document IDs per CIFF file, and which IDs they get remapped to
        self.docid_mapping: list[dict[int, int]] = [{} for _ in readers]
        self.num_postings_lists = 0
        self.num_docs = 0

    def perform_merge(self):
        with tempfile.TemporaryFile() as tmp_file:
            tmp_writer = CiffWriter(tmp_file)

            header = self.merge_headers()

            tmp_writer.write_documents(self.merge_documents())
            tmp_writer.write_postings_lists(self.merge_postings_lists())

            header.num_docs = self.num_docs
            header.num_postings_lists = self.num_postings_lists

            self.writer.write_header(header)

            tmp_file.seek(0)
            while data := tmp_file.read(DEFAULT_CHUNK_SIZE):
                self.writer.output.write(data)

    def get_headers_per_ciff(self) -> list[Header]:
        return [reader.header for reader in self.readers]

    def get_postings_lists_per_ciff(self) -> list[Iterator[PostingsList]]:
        if self.show_progress:
            return [
                iter(tqdm(reader.read_postings_lists(), position=origin, desc=f'Postings lists [CIFF {origin}]'))
                for origin, reader in enumerate(self.readers)
            ]
        else:
            return [iter(reader.read_postings_lists()) for reader in self.readers]

    def get_documents_per_ciff(self) -> list[Iterator[DocRecord]]:
        if self.show_progress:
            return [
                iter(tqdm(reader.read_documents(), position=origin, desc=f'Documents [CIFF {origin}]'))
                for origin, reader in enumerate(self.readers)
            ]
        else:
            return [iter(reader.read_documents()) for reader in self.readers]

    def merge_headers(self) -> Header:
        headers = self.get_headers_per_ciff()

        result = Header()

        assert len(set(h.version for h in headers)) == 1
        assert len(set(h.total_postings_lists for h in headers)) == 1
        assert len(set(h.total_docs for h in headers)) == 1
        assert len(set(h.total_terms_in_collection for h in headers)) == 1
        assert len(set(h.average_doclength for h in headers)) == 1

        result.version = headers[0].version
        result.total_postings_lists = headers[0].total_postings_lists
        result.total_docs = headers[0].total_docs
        result.total_terms_in_collection = headers[0].total_terms_in_collection
        result.average_doclength = headers[0].average_doclength

        result.num_docs = sum(h.num_docs for h in headers)

        descriptions = '\n'.join(f' {i + 1}. {h.description}' for i, h in enumerate(headers))
        result.description = f'This is a combination of {len(headers)} CIFF files:\n\n{descriptions}'

        return result

    def merge_documents(self) -> Iterable[DocRecord]:
        docs_per_ciff = self.get_documents_per_ciff()

        pq: PriorityQueue[tuple[str, tuple[int, DocRecord]]] = PriorityQueue()

        for origin, iterable in enumerate(docs_per_ciff):
            if (doc := next(iterable, None)) is not None:
                pq.put((doc.collection_docid, (origin, doc)))

        # TODO: reconsider whether we want to start from 1 or 0
        new_index = 0

        while not pq.empty():
            _, (origin, doc) = pq.get()

            self.docid_mapping[origin][doc.docid] = new_index
            doc.docid = new_index

            # TODO: We do not wish to output documents with the same collection ID twice
            # if doc.collection_docid != prev_collection_docid:
            #     prev_collection_docid = doc.collection_docid

            yield doc
            self.num_docs += 1

            if (doc := next(docs_per_ciff[origin], None)) is not None:
                pq.put((doc.collection_docid, (origin, doc)))

            new_index += 1

    def merge_postings_lists(self) -> Iterable[PostingsList]:
        postings_lists_per_ciff = self.get_postings_lists_per_ciff()

        pq: PriorityQueue[tuple[str, tuple[int, PostingsList]]] = PriorityQueue()

        for origin, iterable in enumerate(postings_lists_per_ciff):
            if (pl := next(iterable, None)) is not None:
                pq.put((pl.term, (origin, pl)))

        while postings_lists := self.get_top_items(pq):
            result = PostingsList()
            result.term = postings_lists[0][1].term
            # TODO: this will not be as straightforward when relieving the assumption of disjoint document sets
            result.df = sum(pl.df for _, pl in postings_lists)
            result.cf = sum(pl.cf for _, pl in postings_lists)
            result.postings.extend(self.merge_postings({origin: iter(pl.postings) for origin, pl in postings_lists}))

            yield result
            self.num_postings_lists += 1

            for origin, _ in postings_lists:
                if (pl := next(postings_lists_per_ciff[origin], None)) is not None:
                    pq.put((pl.term, (origin, pl)))

    def merge_postings(self, iterables: dict[int, Iterator[Posting]]) -> Iterable[Posting]:
        pq: PriorityQueue[tuple[int, tuple[int, int, Posting]]] = PriorityQueue()

        for origin, iterable in iterables.items():
            if (posting := next(iterable, None)) is not None:
                new_docid = self.docid_mapping[origin][posting.docid]
                pq.put((new_docid, (origin, posting.docid, posting)))

        previous_full_docid = 0
        while not pq.empty():
            full_docid, (origin, previous_docid, posting) = pq.get()

            # D-gap encode the document identifiers
            posting.docid = full_docid - previous_full_docid
            previous_full_docid = full_docid

            yield posting

            if (posting := next(iterables[origin], None)) is not None:
                old_docid = previous_docid + posting.docid
                new_docid = self.docid_mapping[origin][old_docid]
                pq.put((new_docid, (origin, old_docid, posting)))

    @staticmethod
    def get_top_items(pq: PriorityQueue[tuple[str, TA]]) -> list[TA]:
        if pq.empty():
            return []

        first_priority, data = pq.get()
        items = [data]

        while not pq.empty():
            priority, data = pq.get()

            if priority == first_priority:
                items.append(data)
            else:
                pq.put((priority, data))
                break

        return items


def merge_ciff_files(inputs, output, show_progress: bool = True):
    with ExitStack() as stack:
        readers = [stack.enter_context(CiffReader(input_file)) for input_file in inputs]
        writer = stack.enter_context(CiffWriter(output))

        merger = CiffMerge(writer, *readers, show_progress=show_progress)

        merger.perform_merge()



import gzip
from pathlib import Path
from typing import (IO, Callable, Iterable, Iterator, Optional, Sized, TypeVar,
                    Union)

from google.protobuf.internal.decoder import _DecodeVarint32  # type: ignore
from google.protobuf.message import Message

from ciff_toolkit.ciff_pb2 import DocRecord, Header, PostingsList

MT = TypeVar('MT', bound=Message)
T = TypeVar('T')

DEFAULT_CHUNK_SIZE = 128 * 1024


class SizedIterable(Iterable[T], Sized):
    def __init__(self, it: Iterable[T], length: int):
        self.it = it
        self.length = length

    def __len__(self) -> int:
        return self.length

    def __iter__(self) -> Iterator[T]:
        yield from self.it


class BufferedReader:
    filename: Optional[Path] = None
    chunk_reader: Optional[Iterator[bytes]] = None
    fp: Optional[IO[bytes]] = None

    def __init__(self, source: Union[str, Path, Iterable[bytes]]) -> None:
        if isinstance(source, (str, Path)):
            self.filename = Path(source)
        else:
            self.chunk_reader = iter(source)

        self.buffer = bytearray()

    def read_varint(self) -> int:
        self._fill_buffer(16)

        value, offset = _DecodeVarint32(self.buffer, 0)
        self.buffer = self.buffer[offset:]

        return value

    def read(self, n: int) -> bytes:
        self._fill_buffer(n)

        data = self.buffer[:n]
        self.buffer = self.buffer[n:]

        return bytes(data)

    def _fill_buffer(self, n: int) -> None:
        if self.chunk_reader is None:
            raise ValueError('cannot read from non-existent chunk reader')

        while len(self.buffer) < n:
            try:
                self.buffer += next(self.chunk_reader)
            except StopIteration:
                break

    def _file_chunks(self) -> Iterator[bytes]:
        if self.fp is None:
            raise ValueError('cannot read from non-existent file pointer')

        while data := self.fp.read(DEFAULT_CHUNK_SIZE):
            yield data

    def __enter__(self):
        if self.filename is not None:
            open_fn = gzip.open if self.filename.suffix == '.gz' else open
            self.fp = open_fn(self.filename, 'rb')
            self.chunk_reader = self._file_chunks()

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.fp is not None:
            self.fp.close()


class MessageReader(BufferedReader):
    def read_message(self, message_type: Callable[[], MT]) -> MT:
        serialized_message = self.read_serialized()

        message = message_type()
        message.ParseFromString(serialized_message)

        return message

    def read_serialized(self) -> bytes:
        message_size = self.read_varint()
        serialized_message = self.read(message_size)

        return serialized_message


class CiffReader(MessageReader):
    def __init__(self, source: Union[str, Path, Iterable[bytes]]):
        super().__init__(source)

        self._header: Optional[Header] = None

    @property
    def header(self) -> Header:
        if self._header is None:
            return self.read_header()

        return self._header

    def read_header(self) -> Header:
        self._header = self.read_message(Header)
        return self._header

    def read_documents(self) -> Iterable[DocRecord]:
        documents = (self.read_message(DocRecord) for _ in range(self.header.num_docs))
        return SizedIterable(documents, self.header.num_docs)

    def read_serialized_documents(self) -> Iterable[bytes]:
        serialized_documents = (self.read_serialized() for _ in range(self.header.num_docs))
        return SizedIterable(serialized_documents, self.header.num_docs)

    def read_postings_lists(self) -> Iterable[PostingsList]:
        postings_lists = (self.read_message(PostingsList) for _ in range(self.header.num_postings_lists))
        return SizedIterable(postings_lists, self.header.num_postings_lists)

    def read_serialized_postings_lists(self) -> Iterable[bytes]:
        serialized_postings_lists = (self.read_serialized() for _ in range(self.header.num_postings_lists))
        return SizedIterable(serialized_postings_lists, self.header.num_postings_lists)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ciff.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nciff.proto\x12\x0eio.osirrc.ciff\"\xcc\x01\n\x06Header\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x1a\n\x12num_postings_lists\x18\x02 \x01(\x05\x12\x10\n\x08num_docs\x18\x03 \x01(\x05\x12\x1c\n\x14total_postings_lists\x18\x04 \x01(\x05\x12\x12\n\ntotal_docs\x18\x05 \x01(\x05\x12!\n\x19total_terms_in_collection\x18\x06 \x01(\x03\x12\x19\n\x11\x61verage_doclength\x18\x07 \x01(\x01\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\"$\n\x07Posting\x12\r\n\x05\x64ocid\x18\x01 \x01(\x05\x12\n\n\x02tf\x18\x02 \x01(\x05\"_\n\x0cPostingsList\x12\x0c\n\x04term\x18\x01 \x01(\t\x12\n\n\x02\x64\x66\x18\x02 \x01(\x03\x12\n\n\x02\x63\x66\x18\x03 \x01(\x03\x12)\n\x08postings\x18\x04 \x03(\x0b\x32\x17.io.osirrc.ciff.Posting\"G\n\tDocRecord\x12\r\n\x05\x64ocid\x18\x01 \x01(\x05\x12\x18\n\x10\x63ollection_docid\x18\x02 \x01(\t\x12\x11\n\tdoclength\x18\x03 \x01(\x05\x62\x06proto3')



_HEADER = DESCRIPTOR.message_types_by_name['Header']
_POSTING = DESCRIPTOR.message_types_by_name['Posting']
_POSTINGSLIST = DESCRIPTOR.message_types_by_name['PostingsList']
_DOCRECORD = DESCRIPTOR.message_types_by_name['DocRecord']
Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
  'DESCRIPTOR' : _HEADER,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.Header)
  })
_sym_db.RegisterMessage(Header)

Posting = _reflection.GeneratedProtocolMessageType('Posting', (_message.Message,), {
  'DESCRIPTOR' : _POSTING,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.Posting)
  })
_sym_db.RegisterMessage(Posting)

PostingsList = _reflection.GeneratedProtocolMessageType('PostingsList', (_message.Message,), {
  'DESCRIPTOR' : _POSTINGSLIST,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.PostingsList)
  })
_sym_db.RegisterMessage(PostingsList)

DocRecord = _reflection.GeneratedProtocolMessageType('DocRecord', (_message.Message,), {
  'DESCRIPTOR' : _DOCRECORD,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.DocRecord)
  })
_sym_db.RegisterMessage(DocRecord)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEADER._serialized_start=31
  _HEADER._serialized_end=235
  _POSTING._serialized_start=237
  _POSTING._serialized_end=273
  _POSTINGSLIST._serialized_start=275
  _POSTINGSLIST._serialized_end=370
  _DOCRECORD._serialized_start=372
  _DOCRECORD._serialized_end=443
# @@protoc_insertion_point(module_scope)



import gzip
from pathlib import Path
from typing import IO, Iterable, Optional, Union

from google.protobuf.internal.encoder import _VarintEncoder  # type: ignore
from google.protobuf.message import Message

from ciff_toolkit.ciff_pb2 import DocRecord, Header, PostingsList


class MessageWriter:
    filename: Optional[Path] = None
    output: Optional[IO[bytes]] = None

    def __init__(self, output: Union[str, Path, IO[bytes]]) -> None:
        if isinstance(output, (str, Path)):
            self.filename = Path(output)
        else:
            self.output = output

        self.varint_encoder = _VarintEncoder()

    def write_message(self, message: Message):
        self.write_serialized(message.SerializeToString())

    def write_serialized(self, serialized_message: bytes):
        if self.output is None:
            raise ValueError('cannot write to closed file')

        self.varint_encoder(self.output.write, len(serialized_message))
        self.output.write(serialized_message)

    def write(self, data: bytes):
        if self.output is None:
            raise ValueError('cannot write to closed file')

        self.output.write(data)

    def __enter__(self):
        if self.filename is not None:
            open_fn = gzip.open if self.filename.suffix == '.gz' else open
            self.output = open_fn(self.filename, 'wb')

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.filename is not None:
            self.output.close()


class CiffWriter(MessageWriter):
    def write_header(self, header: Header):
        self.write_message(header)

    def write_documents(self, documents: Iterable[DocRecord]):
        for doc in documents:
            self.write_message(doc)

    def write_postings_lists(self, postings_lists: Iterable[PostingsList]):
        for pl in postings_lists:
            self.write_message(pl)



import tempfile
from pathlib import Path
from typing import Iterable

from ciff_toolkit.read import CiffReader, DEFAULT_CHUNK_SIZE
from ciff_toolkit.write import CiffWriter, MessageWriter


def dump_ciff(file: Path):
    with CiffReader(file) as reader:
        print(reader.header)

        for pl in reader.read_postings_lists():
            print(f'{pl.term}\tdf: {pl.df}\tcf: {pl.cf}')

        for doc in reader.read_documents():
            print(f'Doc {doc.docid} ({doc.collection_docid}), length={doc.doclength}')


def _swap_messages(first_messages: Iterable[bytes], second_messages: Iterable[bytes],
                   writer: MessageWriter):
    with tempfile.TemporaryFile() as tmp_file, MessageWriter(tmp_file) as tmp_writer:
        for message in first_messages:
            tmp_writer.write_serialized(message)

        for message in second_messages:
            writer.write_serialized(message)

        tmp_file.seek(0)
        while data := tmp_file.read(DEFAULT_CHUNK_SIZE):
            writer.write(data)


def ciff_swap_docrecords_and_postingslists(file_in: Path, file_out: Path):
    with CiffReader(file_in) as reader, CiffWriter(file_out) as writer:
        writer.write_header(reader.read_header())
        _swap_messages(reader.read_serialized_documents(), reader.read_serialized_postings_lists(), writer)


def ciff_swap_postingslists_and_docrecords(file_in: Path, file_out: Path):
    with CiffReader(file_in) as reader, CiffWriter(file_out) as writer:
        writer.write_header(reader.read_header())
        _swap_messages(reader.read_serialized_postings_lists(), reader.read_serialized_documents(), writer)


def ciff_to_zero_indexed_docids(file_in: Path, file_out: Path):
    with CiffReader(file_in) as reader, CiffWriter(file_out) as writer:
        writer.write_header(reader.read_header())

        for pl in reader.read_postings_lists():
            pl.postings[0].docid -= 1
            writer.write_message(pl)

        for doc in reader.read_documents():
            doc.docid -= 1
            writer.write_message(doc)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ciff_toolkit']

package_data = \
{'': ['*'], 'ciff_toolkit': ['protos/*']}

install_requires = \
['protobuf>=4.21.12,<5.0.0', 'tqdm>=4.64.1,<5.0.0']

entry_points = \
{'console_scripts': ['ciff_dump = ciff_toolkit.cli:dump',
                     'ciff_merge = ciff_toolkit.cli:merge',
                     'ciff_swap = ciff_toolkit.cli:swap',
                     'ciff_zero_index = ciff_toolkit.cli:zero_index']}

setup_kwargs = {
    'name': 'ciff-toolkit',
    'version': '0.1.1',
    'description': 'Toolkit for working with Common Index File Format (CIFF) files.',
    'long_description': "# CIFF Toolkit\n\nThis repository contains a Python toolkit for working with [Common Index File Format (CIFF)](https://github.com/osirrc/ciff/) files.\n\nSpecifically, it provides a `CiffReader` and `CiffWriter` for easily reading and writing CIFF files. It also provides a handful of CLI tools, such as merging a CIFF file or dumping its contents.\n\n## Installation\n\nTo use the CIFF toolkit, install it from PyPI:\n\n```bash\n$ pip install ciff-toolkit\n```\n\n## Usage\n\n### Reading\n\nTo read a CIFF file, you can use the `CiffReader` class. It returns the posting lists and documents as lazy generators, so operations that need to process large CIFF files do not need to load the entire index into memory.\n\nThe `CiffReader` can be used as a context manager, automatically opening files if a path is supplied as a `str` or `pathlib.Path`. \n\n```python\nfrom ciff_toolkit.read import CiffReader\n\nwith CiffReader('./path/to/index.ciff') as reader:\n    header = reader.read_header()\n\n    for pl in reader.read_postings_lists():\n        print(pl)\n\n    for doc in reader.read_documents():\n        print(doc)\n```\n\nAlternatively, the `CiffReader` also accepts iterables of bytes instead of file paths. This could be useful if, for instance, the index is in a remote location:\n\n```python\nimport requests\nfrom ciff_toolkit.read import CiffReader\n\nurl = 'https://example.com/remote-index.ciff'\nwith CiffReader(requests.get(url, stream=True).iter_content(1024)) as reader:\n    header = reader.read_header()\n    ...\n```\n\n### Writing\n\nThe `CiffWriter` offers a similar context manager API:\n\n```python\nfrom ciff_toolkit.ciff_pb2 import Header, PostingsList, DocRecord\nfrom ciff_toolkit.write import CiffWriter\n\nheader: Header = ...\npostings_lists: list[PostingsList] = ...\ndoc_records: list[DocRecord] = ...\n\nwith CiffWriter('./path/to/index.ciff') as writer:\n    writer.write_header(header)\n    writer.write_postings_lists(postings_lists)\n    writer.write_documents(doc_records)\n```\n\n### Command Line Interface\n\nA couple of CLI commands are supported:\n\n- `ciff_dump INPUT`\n\n  Dumps the contents of a CIFF file, in order to inspect its contents.\n- `ciff_merge INPUT... OUTPUT`\n\n  Merges two or more CIFF files into a single CIFF file. Ensures documents and terms are ordered correctly, and will read and write in a streaming manner (i.e. not read all data into memory at once).\n\n  Note: `ciff_merge` requires that the `DocRecord` messages occur before the `PostingsList` messages in the CIFF file, as it needs to remap the internal document identifiers before merging the posting lists. See `ciff_swap` below for more information on how to achieve that. \n- `ciff_swap --input-order [hpd|hdp] INPUT OUTPUT`\n\n  Swaps the `PostingsList` and `DocRecord` messages in a CIFF file (e.g. in order to prepare for merging). The `--input-order` argument specifies the current format of the CIFF file: `hpd` for header - posting lists - documents, and `hdp` for header - documents - posting lists.\n- `ciff_zero_index INPUT OUTPUT`\n\n  Takes a CIFF file with 1-indexed documents, and turns it into 0-indexed documents.\n\n## Development\n\nThis project uses [Poetry](https://python-poetry.org/) to manage dependencies, configure the project and publish it to PyPI.\n\nTo get started, use Poetry to install all dependencies:\n\n```bash\n$ poetry install\n```\n\nThen, either activate the virtual environment to execute all Python code in the virtual environment, or prepend every command with poetry run.\n\n```bash\n$ poetry shell\n(venv) $ ciff_dump index.ciff\n```\n\nor:\n\n```bash\n$ poetry run ciff_dump index.ciff\n```\n",
    'author': 'Gijs Hendriksen',
    'author_email': 'g.hendriksen@cs.ru.nl',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://opencode.it4i.eu/openwebsearcheu-public/ciff-toolkit/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from argparse import ArgumentParser
from pathlib import Path

from ciff_toolkit.merge import merge_ciff_files
from ciff_toolkit.util import (ciff_swap_docrecords_and_postingslists,
                               ciff_swap_postingslists_and_docrecords,
                               ciff_to_zero_indexed_docids, dump_ciff)


def merge():
    parser = ArgumentParser(description='Merge CIFF files into a joined CIFF file.')
    parser.add_argument('inputs', help='Input CIFF files that should be merged', nargs='+', type=Path)
    parser.add_argument('output', help='The destination of the merged CIFF file', type=Path)

    args = parser.parse_args()

    merge_ciff_files(args.inputs, args.output)


def swap():
    parser = ArgumentParser(description='Swap records within a CIFF file.')
    parser.add_argument('input', help='Input CIFF file, of which the records should be swapped', type=Path)
    parser.add_argument('output', help='The destination of the new CIFF file', type=Path)
    parser.add_argument('-o', '--input-order', choices=['hpd', 'hdp'], default='hpd',
                        help='The order of records in the input CIFF file. Options: '
                             '"hpd" (default) for header - posting lists - documents, and '
                             '"hdp" for header - documents - posting lists')

    args = parser.parse_args()

    if args.input_order == 'hpd':
        ciff_swap_postingslists_and_docrecords(args.input, args.output)
    elif args.input_order == 'hdp':
        ciff_swap_docrecords_and_postingslists(args.input, args.output)


def zero_index():
    parser = ArgumentParser(description='Transform 1-indexed docids in a CIFF file to 0-indexed docids.')
    parser.add_argument('input', help='Input CIFF file, of which the docids should be adjusted', type=Path)
    parser.add_argument('output', help='The destination of the new CIFF file', type=Path)

    args = parser.parse_args()

    ciff_to_zero_indexed_docids(args.input, args.output)


def dump():
    parser = ArgumentParser(description='Dump the contents of a CIFF file.')
    parser.add_argument('input', help='Input CIFF file', type=Path)

    args = parser.parse_args()

    dump_ciff(args.input)
